==> recursion_iterator-0.0.2.tar.gz: lib/recursion_iterator.dart <==
library recursion_iterator;

import 'dart:collection';

class IteratorConstance {
  const IteratorConstance();
}

const IteratorConstance _StopIterator = const IteratorConstance();

class RecursionIterator<T> extends Iterator<T> {

  dynamic Function(RecursionIterator<T>) next;
  Queue<Iterator<T>> queue = Queue();
  T _current;

  RecursionIterator(this.next);
  RecursionIterator.expand(Iterable<Iterator<T>> iterable) {
    var iterator = iterable.iterator;
    this.next = (_) {
      if (iterator.moveNext()) {
        return iterator.current;
      } else {
        return _StopIterator;
      }
    };
  }

  @override
  T get current => queue.last?.current ?? _current;

  @override
  bool moveNext() {
    if (queue.length > 0) {
      return _deepNext();
    } else {
      var result = next(this);
      if (result == _StopIterator) {
        return false;
      } else if (result is Iterator<T>) {
        queue.add(result);
        return _deepNext();
      } else if (result is Iterable<T>) {
        queue.add(result.iterator);
        return _deepNext();
      } else {
        _current = result;
        return true;
      }
    }
  }

  bool _deepNext() {
    var iter = queue.last;
    var has = iter.moveNext();
    if (!has) {
      queue.removeLast();
      return moveNext();
    } else {
      return true;
    }
  }

  IteratorConstance get stop => _StopIterator;
  Iterator<T> iterator(Iterator<T> iterator) => iterator;
  Iterable<T> iterable(Iterable<T> iterable) => iterable;
}

class RecursionIterable<T> extends Iterable<T> {
  Iterator<T> Function() init;
  RecursionIterable(this.init);

  @override
  Iterator<T> get iterator => init();
}

==> recursion_iterator-0.0.3.tar.gz: lib/recursion_iterator.dart <==
library recursion_iterator;

import 'dart:collection';

class IteratorConstance {
  const IteratorConstance();
}

const IteratorConstance _StopIterator = const IteratorConstance();

class RecursionIterator<T> extends Iterator<T> {

  dynamic Function(RecursionIterator<T>) next;
  Queue<Iterator<T>> queue = Queue();
  T _current;

  RecursionIterator(this.next);
  RecursionIterator.expand(Iterable<Iterable<T>> iterable) {
    var iterator = iterable.iterator;
    this.next = (_) {
      if (iterator.moveNext()) {
        return iterator.current;
      } else {
        return _StopIterator;
      }
    };
  }

  @override
  T get current => queue.last?.current ?? _current;

  @override
  bool moveNext() {
    if (queue.length > 0) {
      return _deepNext();
    } else {
      var result = next(this);
      if (result == _StopIterator) {
        return false;
      } else if (result is Iterator<T>) {
        queue.add(result);
        return _deepNext();
      } else if (result is Iterable<T>) {
        queue.add(result.iterator);
        return _deepNext();
      } else {
        _current = result;
        return true;
      }
    }
  }

  bool _deepNext() {
    var iter = queue.last;
    var has = iter.moveNext();
    if (!has) {
      queue.removeLast();
      return moveNext();
    } else {
      return true;
    }
  }

  IteratorConstance get stop => _StopIterator;
  Iterator<T> iterator(Iterator<T> iterator) => iterator;
  Iterable<T> iterable(Iterable<T> iterable) => iterable;
}

class RecursionIterable<T> extends Iterable<T> {
  Iterator<T> Function() init;
  RecursionIterable(this.init);

  @override
  Iterator<T> get iterator => init();
}
